import { Message } from 'eris';
import { Client, Command } from '../class';
import { Misc as MiscPBXActions } from '../pbx';

export default class Intercom extends Command {
  constructor(client: Client) {
    super(client);
    this.name = 'intercom';
    this.description = 'Will synthesize inputted text to a recording and dial an intercom to the extension specified, then play the recording.';
    this.usage = `${this.client.config.prefix}intercom <extension> <text>`;
    this.permissions = 1;
    this.guildOnly = true;
    this.enabled = true;
  }

  public async run(message: Message, args: string[]) {
    try {
      if (!args[0]) return this.client.commands.get('help').run(message, [this.name]);
      const loading = await this.loading(message.channel, 'Synthesizing text...');

      const recordingLocation = await MiscPBXActions.TTS(this.client.util.pbx, `Hello, this is the Library of Code Private Branch Exchange dialing you at the request of ${message.author.username} to deliver you a message. Playing message: ${args.slice(1).join(' ')}`, 'MALE');
      await loading.edit(`***${this.client.util.emojis.LOADING} Preparing to dial...***`);
      const channel = await this.client.util.ari.channels.originate({
        endpoint: `PJSIP/${args[0]}`,
        extension: args[0],
        callerId: `TTS PAGE FRM ${message.author.username} <00>`,
        priority: 1,
        app: 'cr-zero',
        variables: {
          'PJSIP_HEADER(add,Call-Info)': '<uri>;answer-after=0',
          'PJSIP_HEADER(add,Alert-Info)': 'Ring Answer',
        },
      });
      await loading.edit(`***${this.client.util.emojis.LOADING} Dialing call...***`);
      channel.once('StasisStart', async (_, chan) => {
        chan.answer();
        await loading.edit(`***${this.client.util.emojis.LOADING} Answer received, starting playback...***`);
        const playback = await chan.play({
          media: ['sound:beep', recordingLocation],
        }, undefined);
        playback.once('PlaybackFinished', async () => {
          chan.hangup();
          await loading.edit(`***${this.client.util.emojis.LOADING} Successfully delivered intercom message to EXT \`${args[0]}\`.***`);
        });
      });
      return undefined;
    } catch (err) {
      return this.client.util.handleError(err, message, this, false);
    }
  }
}
